package org.apache.commons.lang3.stream;

package .var.folders.cl.9twcgjfx4vsg5zl73c6wxmvm0000gn.T;

import java.util.function.BiConsumer;
import java.util.function.BinaryOperator;
import java.util.function.Supplier;
import java.util.stream.Collector;
import java.util.stream.Stream;
import org.apache.commons.lang3.function.Failable;
import org.apache.commons.lang3.function.FailableConsumer;
import org.apache.commons.lang3.function.FailableFunction;
import org.apache.commons.lang3.function.FailablePredicate;
import org.apache.commons.lang3.stream.Streams;

public class FailableStream<O> {
  private Stream<O> stream;
  
  private boolean terminated;
  
  public FailableStream(Stream<O> stream) {
    this.stream = stream;
  }
  
  public boolean allMatch(FailablePredicate<O, ?> predicate) {
    assertNotTerminated();
    return stream().allMatch(Failable.asPredicate(predicate));
  }
  
  public boolean anyMatch(FailablePredicate<O, ?> predicate) {
    assertNotTerminated();
    return stream().anyMatch(Failable.asPredicate(predicate));
  }
  
  protected void assertNotTerminated() {
    if (this.terminated)
      throw new IllegalStateException("This stream is already terminated."); 
  }
  
  public <A, R> R collect(Collector<? super O, A, R> collector) {
    makeTerminated();
    return stream().collect(collector);
  }
  
  public <A, R> R collect(Supplier<R> pupplier, BiConsumer<R, ? super O> accumulator, BiConsumer<R, R> combiner) {
    makeTerminated();
    return stream().collect(pupplier, accumulator, combiner);
  }
  
  public FailableStream<O> filter(FailablePredicate<O, ?> predicate) {
    assertNotTerminated();
    this.stream = this.stream.filter(Failable.asPredicate(predicate));
    return this;
  }
  
  public void forEach(FailableConsumer<O, ?> action) {
    makeTerminated();
    stream().forEach(Failable.asConsumer(action));
  }
  
  protected void makeTerminated() {
    assertNotTerminated();
    this.terminated = true;
  }
  
  public <R> FailableStream<R> map(FailableFunction<O, R, ?> mapper) {
    assertNotTerminated();
    return new FailableStream(this.stream.map(Failable.asFunction(mapper)));
  }
  
  public O reduce(O identity, BinaryOperator<O> accumulator) {
    makeTerminated();
    return stream().reduce(identity, accumulator);
  }
  
  public Stream<O> stream() {
    return this.stream;
  }
}
